@@ -310,6 +310,20 @@ void ClsNoisyEvolution::update_quantum_state(QuantumStateBase* state) {
     delete buffer;
 }
 
+boost::property_tree::ptree ClsNoisyEvolution::to_ptree() const {
+    boost::property_tree::ptree pt;
+    pt.put("name", "NoisyEvolutionGate");
+    pt.put_child("hamiltonian", _hamiltonian->to_ptree());
+    boost::property_tree::ptree c_ops_pt;
+    for (const GeneralQuantumOperator* c_op : _c_ops) {
+        c_ops_pt.push_back(std::make_pair("", c_op->to_ptree()));
+    }
+    pt.put_child("c_ops", c_ops_pt);
+    pt.put("time", _time);
+    pt.put("dt", _dt);
+    return pt;
+}
+
 // ここからfast
 
 double ClsNoisyEvolution_fast::_find_collapse(QuantumStateBase* prev_state,
@@ -585,3 +599,16 @@ void ClsNoisyEvolution_fast::update_quantum_state(QuantumStateBase* state) {
 
     delete buffer;
 }
+
+boost::property_tree::ptree ClsNoisyEvolution_fast::to_ptree() const {
+    boost::property_tree::ptree pt;
+    pt.put("name", "NoisyEvolutionFastGate");
+    pt.put_child("hamiltonian", _hamiltonian->to_ptree());
+    boost::property_tree::ptree c_ops_pt;
+    for (const GeneralQuantumOperator* c_op : _c_ops) {
+        c_ops_pt.push_back(std::make_pair("", c_op->to_ptree()));
+    }
+    pt.put_child("c_ops", c_ops_pt);
+    pt.put("time", _time);
+    return pt;
+}
