@@ -290,6 +290,13 @@ public:
     virtual std::vector<ITYPE> sampling(
         UINT sampling_count, UINT random_seed) = 0;
 
+    /**
+     * \~japanese-en property treeに変換
+     *
+     * @return ptree
+     */
+    virtual boost::property_tree::ptree to_ptree() const = 0;
+
     /**
      * \~japanese-en 量子回路のデバッグ情報の文字列を生成する
      *
@@ -680,6 +687,16 @@ public:
         random.set_seed(random_seed);
         return this->sampling(sampling_count);
     }
+    virtual boost::property_tree::ptree to_ptree() const override {
+        boost::property_tree::ptree pt;
+        pt.put("name", "QuantumState");
+        pt.put("qubit_count", _qubit_count);
+        pt.put_child(
+            "classical_register", ptree::to_ptree(_classical_register));
+        pt.put_child("state_vector", ptree::to_ptree(std::vector<CPPCTYPE>(
+                                         _state_vector, _state_vector + _dim)));
+        return pt;
+    }
 };
 
 using QuantumState = QuantumStateCpu;
