@@ -40,8 +40,6 @@
  *
  */
 DllExport void X_gate(UINT target_qubit_index, CTYPE* state, ITYPE dim);
-void X_gate_single_unroll(UINT target_qubit_index, CTYPE* state, ITYPE dim);
-void X_gate_single_simd(UINT target_qubit_index, CTYPE* state, ITYPE dim);
 void X_gate_parallel_unroll(UINT target_qubit_index, CTYPE* state, ITYPE dim);
 void X_gate_parallel_simd(UINT target_qubit_index, CTYPE* state, ITYPE dim);
 
@@ -321,10 +319,6 @@ void H_gate_parallel_simd(UINT target_qubit_index, CTYPE* state, ITYPE dim);
  */
 DllExport void CNOT_gate(
     UINT control_qubit_index, UINT target_qubit_index, CTYPE* state, ITYPE dim);
-void CNOT_gate_single_unroll(
-    UINT control_qubit_index, UINT target_qubit_index, CTYPE* state, ITYPE dim);
-void CNOT_gate_single_simd(
-    UINT control_qubit_index, UINT target_qubit_index, CTYPE* state, ITYPE dim);
 void CNOT_gate_parallel_unroll(
     UINT control_qubit_index, UINT target_qubit_index, CTYPE* state, ITYPE dim);
 void CNOT_gate_parallel_simd(
@@ -659,16 +653,10 @@ DllExport void single_qubit_Pauli_rotation_gate(UINT target_qubit_index,
  */
 DllExport void single_qubit_dense_matrix_gate(
     UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_dense_matrix_gate_single_unroll(
-    UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
 void single_qubit_dense_matrix_gate_parallel_unroll(
     UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_dense_matrix_gate_single(
-    UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
 void single_qubit_dense_matrix_gate_parallel(
     UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_dense_matrix_gate_single_simd(
-    UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
 void single_qubit_dense_matrix_gate_parallel_simd(
     UINT target_qubit_index, const CTYPE matrix[4], CTYPE* state, ITYPE dim);
 
@@ -699,12 +687,8 @@ void single_qubit_dense_matrix_gate_parallel_simd(
  */
 DllExport void single_qubit_diagonal_matrix_gate(UINT target_qubit_index,
     const CTYPE diagonal_matrix[2], CTYPE* state, ITYPE dim);
-void single_qubit_diagonal_matrix_gate_single_unroll(UINT target_qubit_index,
-    const CTYPE diagonal_matrix[2], CTYPE* state, ITYPE dim);
 void single_qubit_diagonal_matrix_gate_parallel_unroll(UINT target_qubit_index,
     const CTYPE diagonal_matrix[2], CTYPE* state, ITYPE dim);
-void single_qubit_diagonal_matrix_gate_single_simd(UINT target_qubit_index,
-    const CTYPE diagonal_matrix[2], CTYPE* state, ITYPE dim);
 void single_qubit_diagonal_matrix_gate_parallel_simd(UINT target_qubit_index,
     const CTYPE diagonal_matrix[2], CTYPE* state, ITYPE dim);
 
@@ -734,12 +718,8 @@ void single_qubit_diagonal_matrix_gate_parallel_simd(UINT target_qubit_index,
  */
 DllExport void single_qubit_phase_gate(
     UINT target_qubit_index, CTYPE phase, CTYPE* state, ITYPE dim);
-void single_qubit_phase_gate_single_unroll(
-    UINT target_qubit_index, CTYPE phase, CTYPE* state, ITYPE dim);
 void single_qubit_phase_gate_parallel_unroll(
     UINT target_qubit_index, CTYPE phase, CTYPE* state, ITYPE dim);
-void single_qubit_phase_gate_single_simd(
-    UINT target_qubit_index, CTYPE phase, CTYPE* state, ITYPE dim);
 void single_qubit_phase_gate_parallel_simd(
     UINT target_qubit_index, CTYPE phase, CTYPE* state, ITYPE dim);
 
@@ -776,16 +756,10 @@ void single_qubit_phase_gate_parallel_simd(
 DllExport void single_qubit_control_single_qubit_dense_matrix_gate(
     UINT control_qubit_index, UINT control_value, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_control_single_qubit_dense_matrix_gate_single_unroll(
+void single_qubit_control_single_qubit_dense_matrix_gate_unroll(
     UINT control_qubit_index, UINT control_value, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_control_single_qubit_dense_matrix_gate_parallel_unroll(
-    UINT control_qubit_index, UINT control_value, UINT target_qubit_index,
-    const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_control_single_qubit_dense_matrix_gate_single_simd(
-    UINT control_qubit_index, UINT control_value, UINT target_qubit_index,
-    const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void single_qubit_control_single_qubit_dense_matrix_gate_parallel_simd(
+void single_qubit_control_single_qubit_dense_matrix_gate_simd(
     UINT control_qubit_index, UINT control_value, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
 
@@ -825,19 +799,11 @@ DllExport void multi_qubit_control_single_qubit_dense_matrix_gate(
     const UINT* control_qubit_index_list, const UINT* control_value_list,
     UINT control_qubit_index_count, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void multi_qubit_control_single_qubit_dense_matrix_gate_single_unroll(
+void multi_qubit_control_single_qubit_dense_matrix_gate_unroll(
     const UINT* control_qubit_index_list, const UINT* control_value_list,
     UINT control_qubit_index_count, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void multi_qubit_control_single_qubit_dense_matrix_gate_parallel_unroll(
-    const UINT* control_qubit_index_list, const UINT* control_value_list,
-    UINT control_qubit_index_count, UINT target_qubit_index,
-    const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void multi_qubit_control_single_qubit_dense_matrix_gate_single_simd(
-    const UINT* control_qubit_index_list, const UINT* control_value_list,
-    UINT control_qubit_index_count, UINT target_qubit_index,
-    const CTYPE matrix[4], CTYPE* state, ITYPE dim);
-void multi_qubit_control_single_qubit_dense_matrix_gate_parallel_simd(
+void multi_qubit_control_single_qubit_dense_matrix_gate_simd(
     const UINT* control_qubit_index_list, const UINT* control_value_list,
     UINT control_qubit_index_count, UINT target_qubit_index,
     const CTYPE matrix[4], CTYPE* state, ITYPE dim);
@@ -1252,9 +1218,6 @@ void double_qubit_dense_matrix_gate_simd(UINT target_qubit_index1,
 DllExport void multi_qubit_dense_matrix_gate(
     const UINT* target_qubit_index_list, UINT target_qubit_index_count,
     const CTYPE* matrix, CTYPE* state, ITYPE dim);
-void multi_qubit_dense_matrix_gate_single(const UINT* target_qubit_index_list,
-    UINT target_qubit_index_count, const CTYPE* matrix, CTYPE* state,
-    ITYPE dim);
 void multi_qubit_dense_matrix_gate_parallel(const UINT* target_qubit_index_list,
     UINT target_qubit_index_count, const CTYPE* matrix, CTYPE* state,
     ITYPE dim);
