@@ -16,6 +16,7 @@ void dm_normalize(double squared_norm, CTYPE* state, ITYPE dim) {
     const double normalize_factor = 1. / squared_norm;
     ITYPE state_index_y;
 #ifdef _OPENMP
+    OMPutil::get_inst().set_qulacs_num_threads(dim, 10);
 #pragma omp parallel for
 #endif
     for (state_index_y = 0; state_index_y < loop_dim; ++state_index_y) {
@@ -24,6 +25,9 @@ void dm_normalize(double squared_norm, CTYPE* state, ITYPE dim) {
             state[state_index_y * dim + state_index_x] *= normalize_factor;
         }
     }
+#ifdef _OPENMP
+    OMPutil::get_inst().reset_qulacs_num_threads();
+#endif
 }
 
 void dm_single_qubit_dense_matrix_gate(
@@ -49,6 +53,7 @@ void dm_single_qubit_dense_matrix_gate(
 
     ITYPE state_index_x, state_index_y;
 #ifdef _OPENMP
+    OMPutil::get_inst().set_qulacs_num_threads(dim, 10);
 #pragma omp parallel for private(state_index_x)
 #endif
     for (state_index_y = 0; state_index_y < loop_dim; ++state_index_y) {
@@ -91,6 +96,9 @@ void dm_single_qubit_dense_matrix_gate(
                 ext_matrix[14] * cval_10 + ext_matrix[15] * cval_11;
         }
     }
+#ifdef _OPENMP
+    OMPutil::get_inst().reset_qulacs_num_threads();
+#endif
 }
 
 void dm_multi_qubit_control_single_qubit_dense_matrix_gate(
@@ -121,6 +129,7 @@ void dm_multi_qubit_control_single_qubit_dense_matrix_gate(
 
     ITYPE state_index_x, state_index_y;
 #ifdef _OPENMP
+    OMPutil::get_inst().set_qulacs_num_threads(dim, 10);
 #pragma omp parallel for private(state_index_y)
 #endif
     for (state_index_x = 0; state_index_x < dim; ++state_index_x) {
@@ -187,6 +196,9 @@ void dm_multi_qubit_control_single_qubit_dense_matrix_gate(
                 cval_0 * adjoint_matrix[1] + cval_1 * adjoint_matrix[3];
         }
     }
+#ifdef _OPENMP
+    OMPutil::get_inst().reset_qulacs_num_threads();
+#endif
 
     free(insert_index_list);
 }
@@ -406,6 +418,7 @@ void dm_multi_qubit_dense_matrix_gate(const UINT* target_qubit_index_list,
     }
     free(buffer);
 #else
+    OMPutil::get_inst().set_qulacs_num_threads(dim, 0);
     const UINT thread_count = omp_get_max_threads();
     CTYPE* buffer_list = (CTYPE*)malloc(
         (size_t)(sizeof(CTYPE) * matrix_dim * matrix_dim * thread_count));
@@ -476,6 +489,7 @@ void dm_multi_qubit_dense_matrix_gate(const UINT* target_qubit_index_list,
             }
         }
     }
+    OMPutil::get_inst().reset_qulacs_num_threads();
     free(buffer_list);
 #endif
     free(adjoint_matrix);
@@ -582,11 +596,13 @@ void dm_multi_qubit_control_multi_qubit_dense_matrix_gate(
     }
     free(buffer);
 #else
+    OMPutil::get_inst().set_qulacs_num_threads(dim, 0);
     const UINT thread_count = omp_get_max_threads();
     CTYPE* buffer_list =
         (CTYPE*)malloc((size_t)(sizeof(CTYPE) * matrix_dim * thread_count));
     const ITYPE block_size = dim / thread_count;
     const ITYPE residual = dim % thread_count;
+
 #pragma omp parallel
     {
         UINT thread_id = omp_get_thread_num();
@@ -662,6 +678,7 @@ void dm_multi_qubit_control_multi_qubit_dense_matrix_gate(
             }
         }
     }
+    OMPutil::get_inst().reset_qulacs_num_threads();
     free(buffer_list);
 #endif
     free(adjoint_matrix);
